#!/usr/bin/env python3
import os

from setuptools import setup

# parse the repo name from the git repository

URL = "https://github.com/OpenVoiceOS/ovos-utterance-plugin-cancel"
PLUGIN_CLAZZ = "NevermindPlugin"

AUTHOR, REPO = URL.split(".com/")[-1].split("/")
ADDITIONAL_AUTHORS = ["jarbasai"]
AUTHORS = ADDITIONAL_AUTHORS + [AUTHOR]
PKG = "ovos_utterance_plugin_cancel"

BASEDIR = os.path.abspath(os.path.dirname(__file__))
PKGDIR = os.path.join(BASEDIR, PKG)
UTTERANCE_ENTRY_POINT = f'{REPO.lower()}={PKG}:{PLUGIN_CLAZZ}'


def get_version():
    """ Find the version of the package"""
    version_file = f'{BASEDIR}/ovos_utterance_plugin_cancel/version.py'
    major, minor, build, alpha = (None, None, None, None)
    with open(version_file) as f:
        for line in f:
            if 'VERSION_MAJOR' in line:
                major = line.split('=')[1].strip()
            elif 'VERSION_MINOR' in line:
                minor = line.split('=')[1].strip()
            elif 'VERSION_BUILD' in line:
                build = line.split('=')[1].strip()
            elif 'VERSION_ALPHA' in line:
                alpha = line.split('=')[1].strip()

            if ((major and minor and build and alpha) or
                    '# END_VERSION_BLOCK' in line):
                break
    version = f"{major}.{minor}.{build}"
    if alpha and int(alpha) > 0:
        version += f"a{alpha}"
    return version


def find_resource_files():
    resource_base_dirs = ("locale", "res",)  # Removed "ui"
    package_data = list()
    for res in resource_base_dirs:
        if os.path.isdir(os.path.join(PKGDIR, res)):
            for directory, _, files in os.walk(os.path.join(PKGDIR, res)):
                for f in files:
                    path = os.path.relpath(os.path.join(directory, f), PKGDIR)
                    package_data.append(path)
    return package_data


with open(os.path.join(BASEDIR, "README.md"), "r") as f:
    long_description = f.read()


setup(
    name=REPO,
    description='OpenVoiceOS Utterance Cancel Plugin',
    long_description=long_description,
    version=get_version(),
    author=AUTHOR,
    author_email='jarbasai@mailfence.com',
    url=URL,
    license='apache-2.0',
    packages=[PKG],
    package_data={PKG: find_resource_files()},
    include_package_data=True,
    zip_safe=True,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Text Processing :: Linguistic',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    entry_points={
        'neon.plugin.text': UTTERANCE_ENTRY_POINT
    }
)



# # NEON AI (TM) SOFTWARE, Software Development Kit & Application Development System
# # All trademark and other rights reserved by their respective owners
# # Copyright 2008-2021 Neongecko.com Inc.
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# CONTRIBUTORS  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS;  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE,  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
from os.path import join, dirname, isfile
from typing import List, Optional

from functools import lru_cache
from ovos_plugin_manager.templates.transformers import UtteranceTransformer
from ovos_utils.log import LOG
from ovos_utils.bracket_expansion import expand_options


class NevermindPlugin(UtteranceTransformer):

    def __init__(self, name="ovos-utterance-cancel", priority=15):
        super().__init__(name, priority)

    @lru_cache()
    def get_cancel_words(self, lang="en-us"):
        res_path = join(dirname(__file__), "locale", lang, "cancel.intent")
        if isfile(res_path):
            lines = list()
            with open(res_path) as f:
                for line in f.readlines():
                    if line.startswith("#"):
                        continue
                    lines.extend(expand_options(line))
            return lines
        else:
            LOG.warning(f"cancel.dialog not available for {lang}")
        return []
        
    def transform(self, utterances: List[str], context: Optional[dict] = None) -> (list, dict):
        lang = context.get("lang", "en-us")
        for nevermind in self.get_cancel_words(lang):
            for utterance in utterances:
                if utterance.endswith(nevermind):
                    return [], {"canceled": True, "cancel_word": nevermind}

        return utterances, {}
